/{{id}}:
  ros__parameters:
    # Database URL
    db_url: "mongodb://localhost:27017"
    # you can add here and document any other public parameters for your skill
    # parameter_name: parameter_value...
