:root {
  --ghost-grey: #edede9;
  --main-grey: #6e746d;
  --primary-blue: #00759d;
  --secondary-blue: #81ddf2;
  --light-blue: #6abedb;
  --yellow: #f5cd38;
}
@font-face {
  font-family: 'Montserrat';
  font-style: normal;
  font-weight: 300;
  src: url(fonts/montserrat-300-normal.woff2) format('woff2');
}
/* latin */
@font-face {
  font-family: 'Montserrat';
  font-style: italic;
  font-weight: 300;
  src: url(fonts/montserrat-300-italic.woff2) format('woff2');
}


body {
  overflow: hidden;
  font-family: Montserrat;
  color: var(--main-grey);
  margin: 0;
  height: 800px;
  width: 1280px;
  user-select: none;
  background-color: var(--ghost-grey);
}
.d-none {
  display: none!important;
}
.main-container {
  display: flex;
  flex-direction: column;
  justify-content: center;
  align-items: center;
  padding: 10vh;
  box-sizing: border-box;
  height: 100%;
  width: 100%;
}
.question-text {
  font-weight: 600;
  font-size: 55px;
  height: 35%;
  text-align: center;
  justify-content: center;
  align-items: center;
  display: flex;
}
.buttons-grid {
  width: 100%;
  min-height: 45%;
  display: flex;
  justify-content: center;
  align-items: center;
  flex-wrap: wrap;
  margin-top: 5%;
}
.button-item {
  padding: 30px 50px;
  font-size: 35px;
  border-radius: 10px;
  background-color: var(--primary-blue);
  color: #fff;
  margin: 20px;
}
.button-hover {
  background-color: var(--light-blue);
}
.exit-button {
  position: absolute;
  top: 4%;
  right: 4%;
  padding: 14px 38px;
  font-size: 30px;
  color: var(--main-grey);
  border: 1px solid var(--main-grey);
  border-radius: 10px;
}
.exit-button-hover {
  background-color: var(--main-grey);
  color: white;
}
