<?xml version="1.0"?>
<package format="3">
    <name>{{ id }}</name>
    <version>0.1.0</version>
    <description>{{ name }}</description>

  <!-- One maintainer tag required, multiple allowed, one person per tag -->
  <!-- Example:  -->
  <!-- <maintainer email="jane.doe@example.com">Jane Doe</maintainer> -->
  <maintainer email="todo@todo.todo">TODO</maintainer>

  <!-- One license tag required, multiple allowed, one license per tag -->
  <!-- Commonly used license strings: -->
  <!--   BSD, MIT, Boost Software License, GPLv2, GPLv3, LGPLv2.1, LGPLv3 -->
  <license>Apache-2.0</license>

  <!-- Url tags are optional, but multiple are allowed, one per tag -->
  <!-- Optional attribute type can be: website, bugtracker, or repository -->
  <!-- Example: -->
  <!-- <url type="website">http://wiki.ros.org/test</url> -->


  <!-- Author tags are optional, multiple are allowed, one per tag -->
  <!-- Authors do not have to be maintainers, but could be -->
  <!-- Example: -->
  <!-- <author email="jane.doe@example.com">Jane Doe</author> -->


  <!-- The *depend tags are used to specify dependencies -->
  <!-- Dependencies can be catkin packages or system dependencies -->
  <depend>rclpy</depend>
  <depend>hri_actions_msgs</depend>
  <depend>lifecycle_msgs</depend>
  <depend>ament_index_python</depend>

  <!-- As an example, we are going to use the TTS and play_motion -->
  <!-- actions. We need to list the corresponding ROS actions as -->
  <!-- dependencies. Similary, we will publish in the touchscreen, -->
  <!-- so we need to add dependencies to the pal_web_msgs topic -->
  <depend>tts_msgs</depend>
  <depend>play_motion2_msgs</depend>
  <!-- <depend>pal_web_msgs</depend> -->  <!-- Waiting for ros 2 porting -->

  <!-- If we wanted to use navigation, we would likely need nav2_msgs and/or twist_mux_msgs-->
  <!-- You can check the msgs that each interface needs i.e. -->
  <!-- by pressing "documentation" on the skills documentation i.e.-->
  <!-- https://docs.pal-robotics.com/sdk-dev/navigation/skills_list-->
  <depend>nav2_msgs</depend>
  <depend>twist_mux_msgs</depend>

{% if robot == "TIAGO" %}
  <!-- If we wanted to use manipulation for grasping for example-->
  <!-- using TIAGo's advanced manipulation (premium software, ask-->
  <!-- customer support if interested), we would need-->
  <!-- pal_bt_grasping_msgs, as indicated in here:-->
  <!-- https://docs.pal-robotics.com/sdk-dev/manipulation/skills_list-->
  <depend>pal_bt_grasping_msgs</depend>
{% endif %}

  <test_depend>ament_copyright</test_depend>
  <test_depend>ament_flake8</test_depend>
  <test_depend>ament_pep257</test_depend>
  <test_depend>python3-pytest</test_depend>

  <export>
      <build_type>ament_python</build_type>
  </export>
</package>
